@@ -190,6 +190,9 @@ void init_annotation(py::module_ &m);
 void init_page(py::module_ &m);
 size_t page_index(QPDF &owner, QPDFObjectHandle page);
 
+// From rectangle.cpp
+void init_rectangle(py::module_ &m);
+
 inline char *fix_pypy36_const_char(const char *s)
 {
     // PyPy 7.3.1 (=Python 3.6) has a few functions incorrectly defined as requiring
@@ -198,6 +201,14 @@ inline char *fix_pypy36_const_char(const char *s)
     return const_cast<char *>(s);
 }
 
+inline void deprecation_warning(const char *msg)
+{
+    py::object warn = py::module_::import("warnings").attr("warn");
+    py::object DeprecationWarning =
+        py::module_::import("builtins").attr("DeprecationWarning");
+    warn(msg, DeprecationWarning, /*stackleverl=*/1);
+}
+
 // Support for recursion checks
 class StackGuard {
 public:
